@@ -449,14 +449,13 @@ int ScriptCompiler::compile(istream *str, char *name, const char *scrname)
 	int maxargs = SCRIPT_MAX_ARGS;
 	const char *err;
 	char *cp = strrchr(scrname, '.');
-	const char *var = cmds->getLast("datafiles");
-	const char *project = cmds->getLast("name");
+//	const char *var = cmds->getLast("datafiles");
 	bool trapflag;
 	int argc, key, tlen, initkey = 0, initcount = 0;
 	unsigned i;
 	unsigned short count, number, total = 0;
 	size_t gvarlen;
-	Event *events, *esave;
+	NamedEvent *events, *esave;
 	Name *script;
 	Line *line, *last;
 	unsigned long addmask, submask, trapmask, mask, cmask = 0;
@@ -480,9 +479,6 @@ int ScriptCompiler::compile(istream *str, char *name, const char *scrname)
 	buffer = (char *)malloc(512);
 	bufsize = 512;
 
-	if(project && !stricmp(project, "bayonne"))
-		project = NULL;
-
 	scrStream = str;
 
 	if(cp && !stricmp(cp, ".mod"))
@@ -626,7 +622,7 @@ first:
 		{
 			if(*token == '~')
 			{
-		                esave = (Event *)alloc(sizeof(Event));
+		                esave = (NamedEvent *)alloc(sizeof(NamedEvent));
                                 esave->name = alloc(token + 1);
                                 esave->line = NULL;
                                 esave->next = events;
@@ -636,7 +632,7 @@ first:
                         }
 			if(*token == '@' || *token == '{')
 			{
-				esave = (Event *)alloc(sizeof(Event));
+				esave = (NamedEvent *)alloc(sizeof(NamedEvent));
 				esave->name = alloc(token + 1);
 				esave->line = NULL;
 				esave->next = events;
@@ -657,7 +653,8 @@ first:
 				break;
 			}
 
-			if(!stricmp(token, "private") || !stricmp(token,
+			if(!cmds->ripple)
+  			  if(!stricmp(token, "private") || !stricmp(token,
 "protected") || !stricmp(token, "public") || !stricmp(token, "program")
 || !stricmp(token, "state") || !stricmp(token, "module"))
 			{
@@ -692,27 +689,27 @@ repname1:
 			}
 
 
-			if(!stricmp(token, "service"))
+			if(!stricmp(token, "service") && !cmds->ripple)
 			{
 				ftoken = "options";
 				pmode = "public";
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "func", 4))
+			if(!strnicmp(token, "func", 4) && !cmds->ripple)
 			{
 				ftoken = "fconst";
 				pmode = "function";
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "proc", 4))
+			if(!strnicmp(token, "proc", 4) && !cmds->ripple)
 			{
 				pmode = "function";
 				goto repname1;
 			}
 
-			if(!stricmp(token, "catch"))
+			if(!stricmp(token, "catch") && !cmds->ripple)
 			{
 				ftoken = "fconst";
 				name = getToken();
@@ -813,7 +810,7 @@ repname1:
 			snprintf(catchname, sizeof(catchname), "use.%s", token);
 			token = catchname;
 		}
-		else if((!stricmp(token, "requires") || !stricmp(token, "import")) && scrStream == (istream *)&scrSource)
+		else if((!stricmp(token, "requires") || !stricmp(token, "import")) && scrStream == (istream *)&scrSource && !cmds->ripple)
 		{
 			token = getToken();
 			if(!token)
@@ -834,22 +831,22 @@ repname1:
 		if(!token)
 			continue;
 
-		if(*token == '%')
+		if(*token == '%' && !cmds->ripple)
 		{
 			pretoken = token;
-			token = "number";
+			token = "expr";
 		}
-		else if(*token == '&')
+		else if(*token == '&' && !cmds->ripple)
 		{
 			pretoken = token;
 			token = "call";
 		}
-		else if(!strnicmp(token, "*::", 3))
+		else if(!strnicmp(token, "*::", 3) && !cmds->ripple)
 		{
 			pretoken = token + 3;
 			token = "call";
 		}
-		else if(strstr(token, "::"))
+		else if(strstr(token, "::") && !cmds->ripple)
 		{
 			pretoken = token;
 			token = "call";
@@ -993,25 +990,29 @@ repname1:
 
 		addmask = submask = 0;
 
-		if(!stricmp(command, "repeat") || !stricmp(command, "for") || !stricmp(command, "do") || !stricmp(command, "foreach"))
+		if(!cmds->ripple)
 		{
-			if(!looplevel)
-				++loopid;
-			++looplevel;
-			line->loop = loopid * 128 + looplevel;
+			if(!stricmp(command, "repeat") || !stricmp(command, "for") || !stricmp(command, "do") || !stricmp(command, "foreach"))
+			{
+				if(!looplevel)
+					++loopid;
+				++looplevel;
+				line->loop = loopid * 128 + looplevel;
+			}
+
+			if(!stricmp(command, "loop") || !strnicmp(command, "loop.", 5))
+			{
+				line->loop = loopid * 128 + looplevel;
+				if(!looplevel)
+				{
+					slog.error("loop: nesting error %s(%d)", filename, lnum);
+					continue;
+				}
+				else
+					--looplevel;
+			}
 		}
 
-		if(!stricmp(command, "loop") || !strnicmp(command, "loop.", 5))
-		{
-			line->loop = loopid * 128 + looplevel;
-			if(!looplevel)
-			{
-				slog.error("loop: nesting error %s(%d)", filename, lnum);
-				continue;
-			}
-			else
-				--looplevel;
-		}
 		memcpy(line->args, &args, sizeof(char *) * argc);
 
 		err = cmds->check(command, line, this);
