@@ -108,6 +108,9 @@ const char *ScriptInterp::getExternal(const char *opt)
 	Line *line;
 	unsigned idx;
 
+	if(!cmd)
+		return NULL;
+
 	if(!stricmp(opt, "script.id"))
 	{
 		cp = getTemp();
@@ -155,7 +158,7 @@ const char *ScriptInterp::getExternal(const char *opt)
 		cp = getTemp();
 		snprintf(cp, symsize, "%d", frame[stack].base);
 	}
-	return NULL;
+	return cmd->getExternal(opt);
 }
 
 void ScriptInterp::initialize(void)
@@ -192,11 +195,7 @@ void ScriptInterp::release(void)
 
 ScriptSymbols *ScriptInterp::getSymbols(const char *id)
 {
-	Thread *th = Thread::get();
-
-	if(th && th == (Thread *)thread)
-		goto top;
-	else if(strchr(id, '.') && session != this)
+	if(strchr(id, '.') && session != this)
 	{
 		if(lock)
 			lock->leaveMutex();
@@ -205,8 +204,7 @@ ScriptSymbols *ScriptInterp::getSymbols(const char *id)
 	}
 	else
 		release();
-
-top:		
+	
 	if(strchr(id, '.'))
 		return dynamic_cast<ScriptSymbols*>(session);
 	if(!frame[stack].local)
@@ -305,7 +303,7 @@ Script::Symbol *ScriptInterp::mapDirect(const char *id, unsigned short size)
 retry:
 	if(!isalpha(*id))
 	{
-		loginvalid(id);
+		logmissing(id, "invalid");
 		return NULL;
 	}		
 
@@ -316,7 +314,7 @@ retry:
 
 		if(!strchr("abcdefghijklmnopqrstuvwxyz01234567890.", tolower(id[count])))
 		{
-			loginvalid(id);
+			logmissing(id, "invalid");
 			return NULL;
 		}
 		++count;
@@ -324,7 +322,7 @@ retry:
 
 	if(count == 64)
 	{
-		loginvalid(id);
+		logmissing(id, "invalid");
 		return NULL;
 	}
 
@@ -558,7 +556,7 @@ bool ScriptInterp::event(const char *name, bool inhereted)
 {
 	char evtname[128];
 	const char *savname = name;
-	Event *evt, *top = frame[stack].script->events;
+	NamedEvent *evt, *top = frame[stack].script->events;
 	unsigned base = frame[stack].base;
 	const char *chkname = name;
 	unsigned current = stack;
@@ -656,6 +654,7 @@ void ScriptInterp::initRuntime(Name *scr)
 bool ScriptInterp::attach(ScriptCommand *cmdref, const char *scrname)
 {
 	Name *scr;
+	char msg[65];
 
 	cmd = cmdref;
 	enterMutex();
@@ -675,9 +674,16 @@ bool ScriptInterp::attach(ScriptCommand *cmdref, const char *scrname)
 	scr = getScript(scrname);
 	if(!scr || scr->access != scrPUBLIC)
 	{
+                snprintf(msg, sizeof(msg), "%s: attach failed", scrname);
+		if(!image->getLast(msg))
+		{
+			image->setValue(msg, "missing");
+			cmd->errlog("missing", msg);
+		}
 		cmd->leaveMutex();
 		leaveMutex();
 		logerror("missing; attach failed", scrname);
+		snprintf(msg, sizeof(msg), "%s: attach failed", scrname);
 		return false;
 	}
 
@@ -849,6 +855,64 @@ exit:
 	return rtn;
 }
 
+void ScriptInterp::ripple(void)
+{
+	char namebuf[256];
+	char *label;
+	Name *scr = frame[stack].script;
+	Line *line;	
+	const char *name = getValue(NULL);
+
+	snprintf(namebuf, sizeof(namebuf), "%s", name);
+
+	label = strchr(namebuf, ':');
+	if(!label)
+	{
+		label = namebuf;
+		goto find;
+	}
+	*(label++) = 0;
+	scr = getScript(namebuf);
+	if(!scr)
+	{
+		logmissing(name, "missing", "script");
+		error("label-missing");
+		return;
+	}	
+
+find:
+	if(!label || !*label)
+	{
+		line = scr->first;
+		goto done;
+	}
+
+	line = scr->first;
+	while(line)
+	{
+		if(!stricmp(line->cmd, "label"))
+			if(!stricmp(line->args[0], label))
+				break;
+		line = line->next;
+	}
+	
+	if(!line)
+	{
+		logmissing(name, "missing", "script");
+		error("label-missing");
+		return;
+	}
+
+done:
+        frame[stack].caseflag = false;
+        frame[stack].script = scr;
+	frame[stack].first = scr->first;
+        frame[stack].line = line;
+        frame[stack].index = 0;
+	frame[stack].mask = scr->mask;
+	updated = true;
+}
+
 bool ScriptInterp::redirect(bool evflag)
 {
 	char namebuf[256];
@@ -872,6 +936,7 @@ bool ScriptInterp::redirect(bool evflag)
 
 	if(!label)
 	{
+		logmissing(label, "missing", "script");
 		error("branch-failed");
 		return true;
 	}
@@ -933,16 +998,19 @@ retry:
 script:
 	if(!scr)
 	{
+		logmissing(label, "missing", "script");
 		error("script-not-found");
 		return true;
 	}
 	if(pvt && scr->access == scrPRIVATE)
 	{
+		logmissing(label, "access", "script");
 		error("script-private");
 		return true;
 	}
 	if(!isfun && !fun && scr->access == scrFUNCTION && scr != frame[stack].script)
 	{
+		logmissing(label, "access", "script");
 		error("script-function");
 		return true;
 	}
@@ -999,6 +1067,8 @@ bool ScriptInterp::redirect(const char *scriptname)
 		frame[stack].mask = getMask();
 		return true;
 	}
+	else
+		logmissing(namebuf, "missing", "script");
 	return false;
 }
 
@@ -1357,24 +1427,32 @@ void ScriptInterp::logerror(const char *msg, const char *scrname)
 		slog.error() << logname << ": " << msg << endl;
 }
 
-void ScriptInterp::loginvalid(const char *sym)
+void ScriptInterp::logmissing(const char *sym, const char *reason, const char *group)
 {
+	char msg[65];
+
         if(*sym == '@' || *sym == '%' || *sym == '&')
                 ++sym;
 
         slog.warn() << logname << ": " << frame[stack].script->filename
-                << "(" << frame[stack].line->lnum << "): "
-                << sym << " invalid" << endl;
-}
+                << "(" << frame[stack].line->lnum << "): " << group << " "
+                << sym << " " << reason << endl;
 
-void ScriptInterp::logmissing(const char *sym)
-{
-        if(*sym == '@' || *sym == '%' || *sym == '&')
-                ++sym;
+	snprintf(msg, sizeof(msg), "%s(%d): %s %s",
+		frame[stack].script->filename, frame[stack].line->lnum, group, sym);
 
-        slog.warn() << logname << ": " << frame[stack].script->filename
-                << "(" << frame[stack].line->lnum << "): "
-                << sym << " undefined" << endl;
+	cmd->enterMutex();
+	if(image->getLast(msg))
+	{
+		cmd->leaveMutex();
+		return;
+	}
+
+	image->setValue(msg, reason);
+	if(!stricmp(reason, "undefined"))
+		reason = "missing";
+	cmd->errlog(reason, msg);
+	cmd->leaveMutex();
 }
 
 bool ScriptInterp::done(void)
