@@ -748,6 +748,11 @@ bool ScriptMethods::scrGoto(void)
 		branching(cp);
 
 	frame[stack].tranflag = false;
+	if(image->isRipple())
+	{
+		ripple();
+		return true;
+	}
 	return redirect(true);
 }
 
