@@ -76,12 +76,10 @@ class __EXPORT ScriptCompiler;
  */
 class __EXPORT Script
 {
-protected:
+public:
 	class __EXPORT Line;
 	class __EXPORT Name;
 
-
-public:
 	typedef bool (ScriptInterp::*Method)(void);
 	typedef const char *(ScriptCommand::*Check)(Line *line, ScriptImage *img);
 	typedef bool (*Cond)(ScriptInterp *interp, const char *v);
@@ -143,7 +141,7 @@ public:
 #pragma pack()
 #endif
 
-protected:
+public:
 	class __EXPORT Line
 	{
 	public:
@@ -159,21 +157,20 @@ protected:
 		unsigned short argc;
 	};
 
-	class __EXPORT Event
+	class __EXPORT NamedEvent
 	{
 	public:
-		Event *next;
+		NamedEvent *next;
 		Line *line;
 		char type;
 		const char *name;
 	};
 
-public:
 	class __EXPORT Name
 	{
 	public:
 		Name *next;
-		Event *events;
+		NamedEvent *events;
 		Line *first, *select;
 		Line *trap[TRAP_BITS];
 		unsigned long mask;
@@ -303,6 +300,7 @@ private:
 	friend class __EXPORT ScriptImage;
 	friend class __EXPORT ScriptInterp;
 	friend class __EXPORT ScriptCompiler;
+	friend class __EXPORT ScriptBinder;
 
 #if defined(__GNUC__) || !defined(__hpux)
 #pragma pack(1)
@@ -328,12 +326,15 @@ private:
 	int keyword_count;
 	int trap_count;
 	unsigned long imask;
-	bool ripple;
 	unsigned dbcount;
 	void *dbc;
 
 protected:
+	bool ripple;
 	unsigned activity;	// activity counter
+	static ScriptCommand *runtime;
+
+	virtual const char *getExternal(const char *opt);
 
 public:
 	/**
@@ -345,6 +346,15 @@ public:
 	 */
 	Method getHandler(const char *keyword);
 
+	/**
+	 * Issue a control event against current image for attached
+	 * modules until claimed.
+	 *
+	 * @param control arguments.
+	 * @return true if processed.
+	 */
+	bool control(char **args);
+
 protected:
 	/**
 	 * Fetch whether the given keyword is valid for constructor.
@@ -432,6 +442,14 @@ public:
 	 */
 	static const char *findKeyword(Line *line, const char *kw);
 
+	/**
+	 * Server level logging interface override.
+	 *
+	 * @param level of log message.
+	 * @param text of message.
+	 */
+	virtual void errlog(const char *level, const char *msg);
+
 protected:
 	/**
 	 * Test current command against a list of valid keywords.
@@ -552,6 +570,7 @@ class __EXPORT ScriptBinder : public Script
 private:
 	friend class __EXPORT ScriptInterp;
 	friend class __EXPORT ScriptCompiler;
+	friend class __EXPORT ScriptCommand;
 
 	static ScriptBinder *first;
 	ScriptBinder *next;
@@ -562,6 +581,7 @@ protected:
 	
 	virtual void attach(ScriptInterp *interp);
 	virtual void detach(ScriptInterp *interp);
+	virtual bool control(ScriptImage *img, char **args);
 	virtual void down(void);
 	virtual const char *use(Line *line, ScriptImage *img);
 
@@ -575,14 +595,16 @@ public:
 
 class __EXPORT ScriptRuntime : public ScriptCommand
 {
-private:
-	friend class __EXPORT ScriptBinder;
-	static ScriptRuntime *runtime;
-
 public:
 	ScriptRuntime();
 };
 
+class __EXPORT ScriptRipple : public ScriptCommand
+{
+public:
+        ScriptRipple();
+};
+
 class __EXPORT ScriptChecks : public ScriptCommand
 {
 public:
@@ -677,6 +699,8 @@ public:
 
 	const char *chkGoto(Line *line, ScriptImage *img);
 
+	const char *chkLabel(Line *line, ScriptImage *img);
+
 	const char *chkCall(Line *line, ScriptImage *img);
 
 	const char *chkReturn(Line *line, ScriptImage *img);
@@ -915,6 +939,22 @@ public:
 	 */
 	virtual Name *getScript(const char *name);
 
+	/**
+	 * Get the command object associated with the image.
+	 *
+	 * @return command object.
+	 */
+	inline ScriptCommand *getCommand(void)
+		{return cmds;};
+
+	/**
+	 * Get the ripple flag for the current image.
+	 *
+	 * @return true if a ripple image.
+	 */
+	bool isRipple(void)
+		{return cmds->ripple;};
+
 	/**
 	 * Fetch list of relational scripts.
 	 *
@@ -1222,9 +1262,8 @@ protected:
         void trap(const char *trapname);
 
 public:
-	void loginvalid(const char *id);
-	void logmissing(const char *id);
-	void logerror(const char *msg, const char *name = NULL);
+	virtual void logmissing(const char *id, const char *level = "undefined", const char *group = "symbol");
+	virtual void logerror(const char *msg, const char *name = NULL);
 
 	Symbol *mapSymbol(const char *id, unsigned short = 0);
 	Symbol *mapDirect(const char *id, unsigned short = 0);
@@ -1240,6 +1279,8 @@ protected:
 	
 	bool redirect(const char *scr);
 
+	void ripple(void);
+
 	bool redirect(bool evflag);
 
 	unsigned long getMask(void);
